//
// A renderer class
//
// (c) 2012 James Beau Wilkinson
//
// Licensed under the GNU General Public License v3+
//

package com.beauwilkinson.demofox;

import java.util.ArrayList;
import java.util.Random;
import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;
import android.opengl.GLSurfaceView;
import android.opengl.GLU;

final class cuber implements GLSurfaceView.Renderer
{

 private static final float SKY_DISTANCE = 250f;

 private static final int LEVEL2_PRISM_SIMULTANEATY = 23;

 private static final float PHONE_FORWARD_SPEED = 0.15f;

 // For when the user collides with an obstacle
 private static final float BOUNCE_BACK = 0.3f;

 private static final float FRUSTUM_FARPLANE_DISTANCE = 1000f;

 private static final int PRISM_SIMULTANEATY = 16;

 private static final float SHIP_RADIUS = 0.575f;

 private static final float START_ALT_PRISMS = -12f;

 private static final int PULSE_DELAY_MS = 167;

 private static final Random RANDOM_GENERATOR = new Random();

 // The pertain to the camera, and will be constant, since we do
 // not allow rotation of the viewing platform about
 private static final float NORMAL_X = 0f;

 private static final float NORMAL_Y = 1f;

 private static final float NORMAL_Z = 0f;

 private static final float MAX_POSITION = 50000;

 // It is not mandatory to set USING_EMULATOR to "true"
 // when using the emulator, but doing so will
 // reduce complexity / increase speed
 private static final boolean USING_EMULATOR = false;

 private static final float EMULATOR_FORWARD_SPEED = 1.25f;

 private static final int EMULATOR_SIMULTANEATY = 11;

 private static final int PRISMS_PER_DIMENSION = 14;

 @Override
 public void onDrawFrame(final GL10 gl)
 {
  gl.glClear(GL10.GL_COLOR_BUFFER_BIT | GL10.GL_DEPTH_BUFFER_BIT);
  gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);
  gl.glEnableClientState(GL10.GL_COLOR_ARRAY);
  gl.glMatrixMode(GL10.GL_MODELVIEW);
  plane(gl);
  boolean allbehind;
  allbehind = true;
  for (int index = 0; index < xstate.size(); ++index)
  {
   allbehind &= (prism(gl, xstate.get(index), ystate.get(index),
    zstate.get(index)));
  }
  if (allbehind)
  {
   if (wrapitup && altdone)
   {
    positionstate = demofoxactivity.getStartDistance();
    startedalt = false;
    wrapitup = false;
    altdone = false;
   }
   reset(false);
  }
  if (positionstate >= START_ALT_PRISMS)
  {
   if (!startedalt)
   {
    startedalt = true;
    reset(true);
   }
   allbehind = true;
   for (int index = 0; index < altxstate.size(); ++index)
   {
    allbehind &= (prism(gl, altxstate.get(index), altystate.get(index),
     altzstate.get(index), true));
   }
   if (allbehind)
   {
    if (!wrapitup)
    {
     reset(true);
    }
    else
    {
     altdone = true;
    }
   }
  }
  if (USING_EMULATOR)
  {
   positionstate = positionstate + EMULATOR_FORWARD_SPEED;
  }
  else
  {
   positionstate = positionstate + PHONE_FORWARD_SPEED;
  }

  if (positionstate > MAX_POSITION)
  {
   wrapitup = true;
  }
 }

 @Override
 public void onSurfaceChanged(final GL10 gl, final int width, final int height)
 {
  gl.glViewport(0, 0, width, height);
  // Set the projection matrix...
  final float ratio = (float) width / height;
  gl.glMatrixMode(GL10.GL_PROJECTION);
  gl.glLoadIdentity();
  gl.glFrustumf(-ratio, ratio, -1, 1, 1f, FRUSTUM_FARPLANE_DISTANCE * ratio);
 }

 @Override
 public void onSurfaceCreated(final GL10 gl, final EGLConfig config)
 {
  // Speed things up generally...
  gl.glDisable(GL10.GL_DITHER);
  gl.glHint(GL10.GL_PERSPECTIVE_CORRECTION_HINT, GL10.GL_FASTEST);
  gl.glClearColor(0, 0, 0, 0);
  gl.glEnable(GL10.GL_CULL_FACE);
  gl.glShadeModel(GL10.GL_SMOOTH);
  gl.glEnable(GL10.GL_DEPTH_TEST);
 }

 protected static void resetState()
 {
  reset(false);
 }

 protected static void setXpos(float pxpos)
 {
  xposition = pxpos;
 }

 protected static float getXpos()
 {
  return xposition;
 }

 protected static void setYpos(float pypos)
 {
  yposition = pypos;
 }

 protected static float getYpos()
 {
  return yposition;
 }

 protected static void setPositionstate(float positionstate)
 {
  cuber.positionstate = positionstate;
 }

 protected static float getPositionstate()
 {
  return positionstate;
 }

 private static void reset(final boolean alternate)
 {
  if (!alternate)
  {
   paging = getPositionstate() - demofoxactivity.getStartDistance();
   xstate = new ArrayList<Integer>();
   ystate = new ArrayList<Integer>();
   zstate = new ArrayList<Integer>();
   for (int index = 0; index < simultaneaty; ++index)
   {
    xstate.add(RANDOM_GENERATOR.nextInt(PRISMS_PER_DIMENSION)
     - PRISMS_PER_DIMENSION / 2);
    ystate.add(RANDOM_GENERATOR.nextInt(PRISMS_PER_DIMENSION)
     - PRISMS_PER_DIMENSION / 2);
    zstate.add(RANDOM_GENERATOR.nextInt(PRISMS_PER_DIMENSION)
     - PRISMS_PER_DIMENSION / 2);
   }
  }
  else
  {
   altpaging = positionstate - demofoxactivity.getStartDistance();
   altxstate = new ArrayList<Integer>();
   altystate = new ArrayList<Integer>();
   altzstate = new ArrayList<Integer>();
   for (int index = 0; index < simultaneaty; ++index)
   {
    // We subtract here to create a signed number centered about
    // 0, ranging from (-1 * VISIBLE_CUBES / 2) to
    // (VISIBLE_CUBES / 2). This reflects our 3D coord. system.
    altxstate.add(RANDOM_GENERATOR.nextInt(PRISMS_PER_DIMENSION)
     - PRISMS_PER_DIMENSION / 2);
    altystate.add(RANDOM_GENERATOR.nextInt(PRISMS_PER_DIMENSION)
     - PRISMS_PER_DIMENSION / 2);
    altzstate.add(RANDOM_GENERATOR.nextInt(PRISMS_PER_DIMENSION)
     - PRISMS_PER_DIMENSION / 2);
   }
  }
 }

 private static void plane(final GL10 gl)
 {
  gl.glLoadIdentity();
  GLU.gluLookAt(gl, 0, 0, 0, 0, 0, 1, NORMAL_X, NORMAL_Y, NORMAL_Z);
  gl.glTranslatef(0f, 0f, SKY_DISTANCE);
  plane.draw(gl);
 }

 private static void pulse()
 {
  touchableglview.vibe(demofoxactivity.getSingleton());
  try
  {
   Thread.sleep(PULSE_DELAY_MS);
  }
  catch (final InterruptedException e)
  {
   // It's OK, this is not a "hard" real time system
  }
  touchableglview.vibe(demofoxactivity.getSingleton());
  try
  {
   Thread.sleep(PULSE_DELAY_MS);
  }
  catch (final InterruptedException ext)
  {
   // It's OK, this is not a "hard" real time system
  }
  touchableglview.vibe(demofoxactivity.getSingleton()); // You - did - it!
 }

 private static boolean cube(final GL10 gl, final int x, final int y,
  final int z, final boolean alternate)
 {
  // Real drawing coordinates from channel no.:
  // float xposition=xstate.get(0)*cube.getPrismCubeWidth();
  // float yposition=ystate.get(0)*cube.getPrismCubeWidth();
  // float zposition=zstate.get(0)* -cube.getPrismCubeWidth() + cuber.paging;

  demofoxactivity.setScore(demofoxactivity.getScore() + 1);
  gl.glLoadIdentity();
  GLU.gluLookAt(gl, getXpos(), getYpos(), positionstate, getXpos(),
   getYpos(), positionstate + 1f, NORMAL_X, NORMAL_Y, 0f);
  float tpaging;
  if (alternate)
  {
   tpaging = altpaging;
  }
  else
  {
   tpaging = paging;
  }

  // Position the cube
  gl.glTranslatef(cube.getPrismCubeWidth() * x, cube.getPrismCubeWidth() * y,
   -cube.getPrismCubeWidth() * z + tpaging);

  // 0.5 is for rounding
  if (positionstate + cube.getPrismCubeWidth() + 0.5 + SHIP_RADIUS >= -cube
   .getPrismCubeWidth() * z + tpaging
   && positionstate + cube.getPrismCubeWidth() - 0.5 - SHIP_RADIUS <= -cube
    .getPrismCubeWidth() * z + tpaging)
  {
   // All prisms will enter at least once... 0.5 is for rounding between
   // integral coordinate system (which enumerates cubes) and the FP
   // coordinate system used by OpenGL.
   if (getXpos() >= x - 0.5 - SHIP_RADIUS && getXpos() <= x + 0.5 + SHIP_RADIUS)
   {
    if (getYpos() >= y - 0.5 - SHIP_RADIUS
     && getYpos() <= y + 0.5 + SHIP_RADIUS)
    {

     // If desired: touchableglview.vibe(mycontext);
     if (demofoxactivity.getHighScore() <= demofoxactivity.getScore())
     {
      pulse();
     }
     demofoxactivity.setScore(0);
     positionstate -= BOUNCE_BACK;
    }
   }
  }
  cube.draw(gl);
  // Is it behind camera pos?
  return -cube.getPrismCubeWidth() * z + tpaging < positionstate;
 }

 private static boolean prism(final GL10 gl, final int x, final int y,
  final int z)
 {
  return prism(gl, x, y, z, false);
 }

 private static boolean prism(final GL10 gl, final int x, final int y,
  final int z, final boolean alternate)
 {
  boolean ret = true;
  ret &= cube(gl, x, y, z, alternate);
  ret &= cube(gl, x, y, z + 1, alternate);
  ret &= cube(gl, x, y, z + 2, alternate);
  return ret;
 }

 private static float xposition = 0f;
 private static float yposition = 0f;
 private static float altpaging = 0f;
 private static ArrayList<Integer> xstate = new ArrayList<Integer>();
 private static ArrayList<Integer> ystate = new ArrayList<Integer>();
 private static ArrayList<Integer> zstate = new ArrayList<Integer>();
 private static ArrayList<Integer> altxstate = new ArrayList<Integer>();
 private static ArrayList<Integer> altystate = new ArrayList<Integer>();
 private static ArrayList<Integer> altzstate = new ArrayList<Integer>();
 private static float paging = 0f;
 protected static boolean startedalt = false;
 private static float positionstate = 0f;
 private static boolean wrapitup = false; // Time to effect wraparound is
                                          // near...
 private static boolean altdone = false; // For wraparound, too
 private static int simultaneaty;

 static
 {
  if (USING_EMULATOR)
  {
   simultaneaty = EMULATOR_SIMULTANEATY;
  }
  else
  {
   if (demofoxactivity.getLevel() == 1)
   {
    simultaneaty = PRISM_SIMULTANEATY;
   }
   else
   {
    simultaneaty = LEVEL2_PRISM_SIMULTANEATY;
   }
  }

  setPositionstate(demofoxactivity.getStartDistance());
  reset(false);
 }
}