//
// D E M O F O X
//
// Navigate an (unseen) spacecraft through an onslaught of obstacle prisms.
//
// (c) 2012 James Beau Wilkinson
//
// Licensed under the GNU General Public License v3+
//

package com.beauwilkinson.demofox;

import java.util.List;

import android.app.Activity;
import android.content.Context;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.TextView;

public final class demofoxactivity extends Activity
{

 private static final float WORLD_LIMIT = 7f;
 private static final float START_DISTANCE = -24f;
 private static int X_ACCELEROMETER = 0;
 private static int Y_ACCELEROMETER = 1;
 private static int Z_ACCELEROMETER = 2;
 private static float X_BACKPLANE_MIN = -8f;
 private static float Y_BACKPLANE_MIN = -8f;
 private static float Y_BACKPLANE_MAX = 8f;
 private static final float YOKE_UI_X_LIMIT = 8f;
 private static final float TILT_SPEED_X = 24f;
 private static final float TILT_SPEED_Y = 41f;
 private static final int MAX_SCORE = 1000000;

 public demofoxactivity()
 {
  super();
  singleton = this;
 }

 @Override
 protected void onCreate(Bundle savedInstanceState)
 {

  super.onCreate(savedInstanceState);

  manager = (SensorManager) getSystemService(Context.SENSOR_SERVICE);

  sensors = manager.getSensorList(Sensor.TYPE_ACCELEROMETER);

  if (sensors.size() > 0)
  {
   acceleration = sensors.get(0);
  }

  manager.registerListener(locallistener, acceleration,
   SensorManager.SENSOR_DELAY_FASTEST);
  LinearLayout fl = new LinearLayout(this);
  fl.setOrientation(LinearLayout.VERTICAL);
  makebuttons(fl);
  setContentView(fl);
 }

 @Override
 protected void onResume()
 {

  LinearLayout fl = new LinearLayout(this);

  fl.setOrientation(LinearLayout.VERTICAL);

  makebuttons(fl);

  setContentView(fl);

  layout();

  super.onResume();
  localsurfaceview.onResume();

  manager.registerListener(locallistener, acceleration,
   SensorManager.SENSOR_DELAY_FASTEST);

 }

 @Override
 protected void onPause()
 {
  clearedinitialization = false;
  clearedinitialization2 = false;
  xbackplane = null;
  ybackplane = null;
  super.onPause();
  localsurfaceview.onPause();

  manager.unregisterListener(locallistener);

 }

 protected static void setScore(int scorex)
 {
  if (scorex > MAX_SCORE)
   return;
  demofoxactivity.score = scorex;
  if (scorex > getHighScore())
   highscore = scorex;
 }

 protected static float getStartDistance()
 {
  return START_DISTANCE;
 }

 protected static int getScore()
 {
  return score;
 }

 protected static int getHighScore()
 {
  return highscore;
 }

 protected static int getLevel()
 {
  return level;
 }

 protected static float getWorldLimit()
 {
  return WORLD_LIMIT;
 }

 protected static demofoxactivity getSingleton()
 {
  return singleton;
 }

 protected static TextView getHighScoreControl()
 {
  return highscoretv;
 }

 protected static TextView getScoreControl()
 {
  return scoretv;
 }

 private static void makebuttons(LinearLayout fl)
 {
  if (clearedinitialization)
  {
   Button temptbtn = new Button(singleton);
   temptbtn.setText(singleton.getString(R.string.level_1));
   View.OnClickListener ox = new buttonhandler();
   temptbtn.setOnClickListener(ox);
   fl.addView(temptbtn);

   temptbtn = new Button(singleton);
   temptbtn.setText(singleton.getString(R.string.level_2));
   ox = new buttonhandler();
   temptbtn.setOnClickListener(ox);
   fl.addView(temptbtn);
   modetv = new TextView(singleton);
   modetv.setText(singleton.getString(R.string.dots));
   fl.addView(modetv);

  }
  else
  {
   Button tempbtn = new Button(singleton);
   tempbtn.setText(singleton.getString(R.string.touchscreen_only));
   View.OnClickListener ox = new buttonhandler();
   tempbtn.setOnClickListener(ox);
   fl.addView(tempbtn);

   tempbtn = new Button(singleton);
   tempbtn.setText(singleton.getString(R.string.accelerometer_ui));
   ox = new buttonhandler();
   tempbtn.setOnClickListener(ox);
   fl.addView(tempbtn);
   modetv = new TextView(singleton);
   modetv.setText(singleton.getString(R.string.dots));
   fl.addView(modetv);
  }
 }

 private static void handlescore()
 {
  if (scoretv != null)
   scoretv.setText((new Integer(demofoxactivity.score)).toString());
  if (highscoretv != null)
   highscoretv.setText(demofoxactivity.singleton.getString(R.string.high_score)
    + (new Integer(demofoxactivity.getHighScore())).toString());
 }

 private static void conform()
 {
  if (cuber.getXpos() > WORLD_LIMIT)
  {
   cuber.setXpos(WORLD_LIMIT);
  }
  if (cuber.getYpos() > WORLD_LIMIT)
  {
   cuber.setYpos(WORLD_LIMIT);
  }
  if (cuber.getXpos() < -WORLD_LIMIT)
  {
   cuber.setXpos(-WORLD_LIMIT);
  }
  if (cuber.getYpos() < -WORLD_LIMIT)
  {
   cuber.setYpos(-WORLD_LIMIT);
  }
 }

 private static void readaccelerometers(SensorEvent event)
 {
  if (yokemode)
   cuber.setYpos(cuber.getYpos() - ytiltfactor
    * ((event.values[Z_ACCELEROMETER] - xbackplane) / TILT_SPEED_Y));
  else
   cuber.setYpos(cuber.getYpos() + ytiltfactor
    * ((event.values[X_ACCELEROMETER] - xbackplane) / TILT_SPEED_Y));

  if (yokemode)
   cuber.setXpos(cuber.getXpos() - xtiltfactor
    * ((event.values[Y_ACCELEROMETER] - ybackplane) / TILT_SPEED_X));
  else
   cuber.setXpos(cuber.getXpos() + xtiltfactor
    * ((event.values[Y_ACCELEROMETER] - ybackplane) / TILT_SPEED_X));
 }

 private static void layout()
 {

  LinearLayout ll = new LinearLayout(singleton);
  ll.setOrientation(LinearLayout.VERTICAL);
  scoretv = new TextView(singleton);
  highscoretv = new TextView(singleton);
  setScore(0);
  cuber.setPositionstate(START_DISTANCE);
  cuber.setXpos(0f);
  cuber.startedalt = false;
  cuber.setYpos(0f);
  cuber.resetState();
  ll.addView(highscoretv);
  ll.addView(scoretv);
  localsurfaceview = new touchableglview(singleton);
  plane.rebuild();
  localsurfaceview.setRenderer(new cuber());
  ll.addView(localsurfaceview);

  if (clearedinitialization2)
  {
   singleton.setContentView(ll);
  }

  return;
 }

 // This (class) member is not static; the infrastructures requires a true
 // instance.
 private final SensorEventListener locallistener = new SensorEventListener()
 {
  @Override
  public void onSensorChanged(SensorEvent event)
  {
   handlescore();
   if (!clearedinitialization2)
   {
    // For setup display (widget GUI)
    float txbackplanelocation;
    float tybackplanelocation;

    txbackplanelocation = event.values[X_ACCELEROMETER];
    tybackplanelocation = event.values[Y_ACCELEROMETER];

    if (txbackplanelocation < X_BACKPLANE_MIN
     || tybackplanelocation > Y_BACKPLANE_MAX
     || tybackplanelocation < Y_BACKPLANE_MIN)
    {
     modetv.setText(demofoxactivity.singleton
      .getString(R.string.not_a_valid_control_position));
    }
    else if (txbackplanelocation > YOKE_UI_X_LIMIT)
    {
     modetv.setText(demofoxactivity.singleton.getString(R.string.yoke_mode));
    }
    else
     modetv.setText(demofoxactivity.singleton.getString(R.string.divecam_mode));
    return;
   }

   if (xbackplane == null)
   {
    xbackplane = event.values[X_ACCELEROMETER];
    ybackplane = event.values[Y_ACCELEROMETER];

    // Don't allow user to proceed in invalid control position
    if ((xbackplane < X_BACKPLANE_MIN || ybackplane > Y_BACKPLANE_MAX || ybackplane < Y_BACKPLANE_MIN)
     && (xtiltfactor > 0.0 || ytiltfactor > 0.0))
    {
     clearedinitialization = false;
     clearedinitialization2 = false;
     xbackplane = null;
     ybackplane = null;
     LinearLayout fl = new LinearLayout(demofoxactivity.singleton);
     fl.setOrientation(LinearLayout.VERTICAL);
     makebuttons(fl);
     setContentView(fl);
     return;
    }

    if (xbackplane > YOKE_UI_X_LIMIT)
    {
     // "Yoke" mode
     xbackplane = event.values[Z_ACCELEROMETER];// Third accelerometer (Z)
     yokemode = true;
    }
    else
    {
     yokemode = false;
    }
    return;
   }
   readaccelerometers(event);
   conform();
  }

  @Override
  public void onAccuracyChanged(Sensor sensor, int accuracy)
  {
   // We have to at least supply an empty override or errors result
  }
 };

 private static class buttonhandler implements View.OnClickListener
 {

  @Override
  public void onClick(View v)
  {
   if (((Button) v).getText().toString()
    .contains(getSingleton().getString(R.string.level_1)))
   {
    level = 1; // Level 2
    clearedinitialization2 = true;
   }
   else if (((Button) v).getText().toString()
    .contains(getSingleton().getString(R.string.level_2)))
   {
    level = 2; // Level 1
    clearedinitialization2 = true;
   }
   else
   {
    if (((Button) v).getText().toString()
     .contains(getSingleton().getString(R.string.accelerometer_ui)))
    {
     xtiltfactor = 1.0f;
     ytiltfactor = 1.0f;
    }
    if (((Button) v).getText().toString()
     .contains(getSingleton().getString(R.string.touchscreen_only)))
    {
     xtiltfactor = 0.0f;
     ytiltfactor = 0.0f;
    }

    clearedinitialization = true;
    LinearLayout fl = new LinearLayout(getSingleton());

    fl.setOrientation(LinearLayout.VERTICAL);

    makebuttons(fl);

    getSingleton().setContentView(fl);

   }

   layout();

  }

 }

 private static float xtiltfactor = 0.0f;
 private static float ytiltfactor = 0.0f;
 private static boolean clearedinitialization = false;
 private static boolean clearedinitialization2 = false;
 private static touchableglview localsurfaceview;
 private static SensorManager manager;
 private static List<Sensor> sensors;
 private static Sensor acceleration;
 private static int level;
 private static demofoxactivity singleton = null;
 private static TextView scoretv = null;
 private static TextView highscoretv = null;
 private static int score = 0;
 private static TextView modetv;
 private static int highscore = 0;
 private static Float xbackplane = null; // For accelerometer UI
 private static Float ybackplane = null;
 private static boolean yokemode = false;

}
