//
// Touchscreen-compatible OpenGL view subclass for Android OS
//
// (c) 2012 James Beau Wilkinson
//
// Licensed under the GNU General Public License v3+
//

package com.beauwilkinson.demofox;

import android.content.Context;
import android.opengl.GLSurfaceView;
import android.os.Vibrator;
import android.view.MotionEvent;

final class touchableglview extends GLSurfaceView
{

 // For touchscreen movements
 private static final float FINENESS = 100f;

 private static int VIBE_MS = 12;

 // Percent of limit to bounce back to when touchscreen user attempts to leave
 // allowed bounding cube
 private static final float BOUNCE_TO = 0.75f;

 protected touchableglview(final Context c)
 {
  super(c);
 }

 @Override
 public boolean onTouchEvent(final MotionEvent event)
 {

  if (demofoxactivity.getScoreControl() != null)
  {

   demofoxactivity.getScoreControl().setText(
    (new Integer(demofoxactivity.getScore())).toString());
  }

  if (demofoxactivity.getHighScoreControl() != null)
  {
   demofoxactivity.getHighScoreControl().setText(
    "HIGH SCORE = " + (new Integer(demofoxactivity.getHighScore())).toString());
  }

  float dy = 0, dx = 0;
  if (event.getAction() == MotionEvent.ACTION_DOWN)
  {
   oldy = event.getY();
   originaly = cuber.getYpos();
   oldx = event.getX();
   originalx = cuber.getXpos();
  }
  else
  {
   dy = event.getY() - oldy;
   cuber.setYpos(originaly + (dy / FINENESS));

   // dragger UI
   dx = event.getX() - oldx;
   cuber.setXpos(originalx + (dx / FINENESS));

   if (cuber.getXpos() > demofoxactivity.getWorldLimit())
   {
    cuber.setXpos(demofoxactivity.getWorldLimit() * BOUNCE_TO);
    vibe(getContext());
   }
   if (cuber.getYpos() > demofoxactivity.getWorldLimit())
   {
    cuber.setYpos(demofoxactivity.getWorldLimit() * BOUNCE_TO);
    vibe(getContext());
   }
   if (cuber.getXpos() < -demofoxactivity.getWorldLimit())
   {
    cuber.setXpos(-demofoxactivity.getWorldLimit() * BOUNCE_TO);
    vibe(getContext());

   }
   if (cuber.getYpos() < -demofoxactivity.getWorldLimit())
   {
    cuber.setYpos(-demofoxactivity.getWorldLimit() * BOUNCE_TO);
    vibe(getContext());
   }

  }

  return true;

 }

 protected static void vibe(final Context ct)
 {

  // An entry is also required in the manifest, e.g.
  // <uses-permission android:name="android.permission.VIBRATE"/>
  //
  final Vibrator v = (Vibrator) ct.getSystemService(Context.VIBRATOR_SERVICE);

  v.vibrate(VIBE_MS);
 }

 private static float oldy = 0f;
 private static float originaly = 0f;
 private static float oldx = 0f;
 private static float originalx = 0f;

}
